'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIX_subsample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIX_subsample',error_checker=_errors._error_checker)
GL_PACK_SUBSAMPLE_RATE_SGIX=_C('GL_PACK_SUBSAMPLE_RATE_SGIX',0x85A0)
GL_PIXEL_SUBSAMPLE_2424_SGIX=_C('GL_PIXEL_SUBSAMPLE_2424_SGIX',0x85A3)
GL_PIXEL_SUBSAMPLE_4242_SGIX=_C('GL_PIXEL_SUBSAMPLE_4242_SGIX',0x85A4)
GL_PIXEL_SUBSAMPLE_4444_SGIX=_C('GL_PIXEL_SUBSAMPLE_4444_SGIX',0x85A2)
GL_UNPACK_SUBSAMPLE_RATE_SGIX=_C('GL_UNPACK_SUBSAMPLE_RATE_SGIX',0x85A1)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_transform_feedback3'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_transform_feedback3',error_checker=_errors._error_checker)
GL_MAX_TRANSFORM_FEEDBACK_BUFFERS=_C('GL_MAX_TRANSFORM_FEEDBACK_BUFFERS',0x8E70)
GL_MAX_VERTEX_STREAMS=_C('GL_MAX_VERTEX_STREAMS',0x8E71)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint)
def glBeginQueryIndexed(target,index,id):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint)
def glDrawTransformFeedbackStream(mode,id,stream):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glEndQueryIndexed(target,index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetQueryIndexediv(target,index,pname,params):pass



#! /usr/bin/env python
"""Python 3.x buffer-handling (currently just for bytes/bytearray types)
"""
import ctypes,sys
if sys.version_info[:2] < (2,6):
    raise ImportError( 'Buffer interface only usable on Python 2.6+' )
from ._arrayconstants import *

PyBUF_SIMPLE = 0
PyBUF_WRITABLE = PyBUF_WRITEABLE = 0x0001
PyBUF_ND = 0x0008
PyBUF_STRIDES = (0x0010 | PyBUF_ND)
PyBUF_CONTIG = (PyBUF_ND | PyBUF_WRITABLE)
PyBUF_CONTIG_RO = (PyBUF_ND)
PyBUF_C_CONTIGUOUS = (0x0020 | PyBUF_STRIDES)
PyBUF_FORMAT = 0x0004

# Python 2.6 doesn't define this...
c_ssize_t = getattr( ctypes, 'c_ssize_t',ctypes.c_ulong)
    
_fields_ = [
    ('buf',ctypes.c_void_p),
    ('obj',ctypes.c_void_p),
    ('len',c_ssize_t),
    ('itemsize',c_ssize_t),

    ('readonly',ctypes.c_int),
    ('ndim',ctypes.c_int),
    ('format',ctypes.c_char_p),
    ('shape',ctypes.POINTER(c_ssize_t)),
    ('strides',ctypes.POINTER(c_ssize_t)),
    ('suboffsets',ctypes.POINTER(c_ssize_t)),
]


if sys.version_info[:2] <= (2,6) or sys.version_info[:2] >= (3,3):
    # Original structure was eventually restored in 3.3, so just 
    # 2.7 through 3.2 uses the "enhanced" structure below
    _fields_.extend( [
        ('internal',ctypes.c_void_p),
    ] )
else:
    # Sigh, this structure seems to have changed with Python 3.x...
    _fields_.extend( [
        ('smalltable',ctypes.c_size_t*2),
        ('internal',ctypes.c_void_p),
    ] )

class Py_buffer(ctypes.Structure):
    """Wrapper around the Python buffer structure..."""
    @classmethod 
    def from_object( cls, object, flags=PyBUF_STRIDES|PyBUF_FORMAT|PyBUF_C_CONTIGUOUS ):
        """Create a new Py_buffer referencing ram of object"""
        if not CheckBuffer( object ):
            raise TypeError( "%s type does not support Buffer Protocol"%(object.__class__,))
        buf = cls()
        # deallocation of the buf causes glibc abort :(
        result = GetBuffer( object, buf, flags )
        if result != 0:
            raise ValueError( "Unable to retrieve Buffer from %s"%(object,))
        if not buf.buf:
            raise ValueError( "Null pointer result from %s"%(object,) )
        return buf
    _fields_ = _fields_
    @property
    def dims( self ):
        return self.shape[:self.ndim]
    def __len__( self ):
        return self.shape[0]
    @property 
    def dim_strides( self ):
        if self.strides:
            return self.strides[:self.ndim]
        return None
    def __enter__(self):
        pass 
    def __exit__( self, exc_type=None, exc_value=None, traceback=None):    
        if self.obj:
            ReleaseBuffer( self )
    def __del__( self ):
        if self.obj:
            ReleaseBuffer( self )
    
BUFFER_POINTER = ctypes.POINTER( Py_buffer )


try:
    CheckBuffer = ctypes.pythonapi.PyObject_CheckBuffer
    CheckBuffer.argtypes = [ctypes.py_object]
    CheckBuffer.restype = ctypes.c_int
except AttributeError as err:
    # Python 2.6 doesn't appear to have CheckBuffer support...
    CheckBuffer = lambda x: True

IncRef = ctypes.pythonapi.Py_IncRef 
IncRef.argtypes = [ ctypes.py_object ]
    
GetBuffer = ctypes.pythonapi.PyObject_GetBuffer
GetBuffer.argtypes = [ ctypes.py_object, BUFFER_POINTER, ctypes.c_int ]
GetBuffer.restype = ctypes.c_int

ReleaseBuffer = ctypes.pythonapi.PyBuffer_Release
ReleaseBuffer.argtypes = [ BUFFER_POINTER ]
ReleaseBuffer.restype = None




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_color_buffer_half_float'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_color_buffer_half_float',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT=_C('GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE_EXT',0x8211)
GL_R16F_EXT=_C('GL_R16F_EXT',0x822D)
GL_RG16F_EXT=_C('GL_RG16F_EXT',0x822F)
GL_RGB16F_EXT=_C('GL_RGB16F_EXT',0x881B)
GL_RGBA16F_EXT=_C('GL_RGBA16F_EXT',0x881A)
GL_UNSIGNED_NORMALIZED_EXT=_C('GL_UNSIGNED_NORMALIZED_EXT',0x8C17)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIS_texture_select'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIS_texture_select',error_checker=_errors._error_checker)
GL_DUAL_ALPHA12_SGIS=_C('GL_DUAL_ALPHA12_SGIS',0x8112)
GL_DUAL_ALPHA16_SGIS=_C('GL_DUAL_ALPHA16_SGIS',0x8113)
GL_DUAL_ALPHA4_SGIS=_C('GL_DUAL_ALPHA4_SGIS',0x8110)
GL_DUAL_ALPHA8_SGIS=_C('GL_DUAL_ALPHA8_SGIS',0x8111)
GL_DUAL_INTENSITY12_SGIS=_C('GL_DUAL_INTENSITY12_SGIS',0x811A)
GL_DUAL_INTENSITY16_SGIS=_C('GL_DUAL_INTENSITY16_SGIS',0x811B)
GL_DUAL_INTENSITY4_SGIS=_C('GL_DUAL_INTENSITY4_SGIS',0x8118)
GL_DUAL_INTENSITY8_SGIS=_C('GL_DUAL_INTENSITY8_SGIS',0x8119)
GL_DUAL_LUMINANCE12_SGIS=_C('GL_DUAL_LUMINANCE12_SGIS',0x8116)
GL_DUAL_LUMINANCE16_SGIS=_C('GL_DUAL_LUMINANCE16_SGIS',0x8117)
GL_DUAL_LUMINANCE4_SGIS=_C('GL_DUAL_LUMINANCE4_SGIS',0x8114)
GL_DUAL_LUMINANCE8_SGIS=_C('GL_DUAL_LUMINANCE8_SGIS',0x8115)
GL_DUAL_LUMINANCE_ALPHA4_SGIS=_C('GL_DUAL_LUMINANCE_ALPHA4_SGIS',0x811C)
GL_DUAL_LUMINANCE_ALPHA8_SGIS=_C('GL_DUAL_LUMINANCE_ALPHA8_SGIS',0x811D)
GL_DUAL_TEXTURE_SELECT_SGIS=_C('GL_DUAL_TEXTURE_SELECT_SGIS',0x8124)
GL_QUAD_ALPHA4_SGIS=_C('GL_QUAD_ALPHA4_SGIS',0x811E)
GL_QUAD_ALPHA8_SGIS=_C('GL_QUAD_ALPHA8_SGIS',0x811F)
GL_QUAD_INTENSITY4_SGIS=_C('GL_QUAD_INTENSITY4_SGIS',0x8122)
GL_QUAD_INTENSITY8_SGIS=_C('GL_QUAD_INTENSITY8_SGIS',0x8123)
GL_QUAD_LUMINANCE4_SGIS=_C('GL_QUAD_LUMINANCE4_SGIS',0x8120)
GL_QUAD_LUMINANCE8_SGIS=_C('GL_QUAD_LUMINANCE8_SGIS',0x8121)
GL_QUAD_TEXTURE_SELECT_SGIS=_C('GL_QUAD_TEXTURE_SELECT_SGIS',0x8125)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_invalidate_subdata'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_invalidate_subdata',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLuint)
def glInvalidateBufferData(buffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glInvalidateBufferSubData(buffer,offset,length):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray)
def glInvalidateFramebuffer(target,numAttachments,attachments):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glInvalidateSubFramebuffer(target,numAttachments,attachments,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint)
def glInvalidateTexImage(texture,level):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glInvalidateTexSubImage(texture,level,xoffset,yoffset,zoffset,width,height,depth):pass



'''OpenGL extension VERSION.GL_4_2

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_4_2 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_4_2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_4_2 import *
from OpenGL.raw.GL.VERSION.GL_4_2 import _EXTENSION_NAME

def glInitGl42VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawElementsInstancedBaseInstance.indices size not checked against count
glDrawElementsInstancedBaseInstance=wrapper.wrapper(glDrawElementsInstancedBaseInstance).setInputArraySize(
    'indices', None
)
# INPUT glDrawElementsInstancedBaseVertexBaseInstance.indices size not checked against count
glDrawElementsInstancedBaseVertexBaseInstance=wrapper.wrapper(glDrawElementsInstancedBaseVertexBaseInstance).setInputArraySize(
    'indices', None
)
glGetInternalformativ=wrapper.wrapper(glGetInternalformativ).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetActiveAtomicCounterBufferiv=wrapper.wrapper(glGetActiveAtomicCounterBufferiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION

from OpenGL.GL.ARB.base_instance import *
from OpenGL.GL.ARB.shading_language_420pack import *
from OpenGL.GL.ARB.transform_feedback_instanced import *
from OpenGL.GL.ARB.compressed_texture_pixel_storage import *
from OpenGL.GL.ARB.conservative_depth import *
from OpenGL.GL.ARB.internalformat_query import *
from OpenGL.GL.ARB.map_buffer_alignment import *
from OpenGL.GL.ARB.shader_atomic_counters import *
from OpenGL.GL.ARB.shader_image_load_store import *
from OpenGL.GL.ARB.shading_language_packing import *
from OpenGL.GL.ARB.texture_storage import *



'''OpenGL extension MESA.pack_invert

This module customises the behaviour of the 
OpenGL.raw.GL.MESA.pack_invert to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a new pixel storage parameter to indicate that
	images are to be packed in top-to-bottom order instead of OpenGL's
	conventional bottom-to-top order.  Only pixel packing can be
	inverted (i.e. for glReadPixels, glGetTexImage, glGetConvolutionFilter,
	etc).
	
	Almost all known image file formats store images in top-to-bottom
	order.  As it is, OpenGL reads images from the frame buffer in
	bottom-to-top order.  Thus, images usually have to be inverted before
	writing them to a file with image I/O libraries.  This extension
	allows images to be read such that inverting isn't needed.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/pack_invert.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.MESA.pack_invert import *
from OpenGL.raw.GL.MESA.pack_invert import _EXTENSION_NAME

def glInitPackInvertMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIS.texture_color_mask

This module customises the behaviour of the 
OpenGL.raw.GL.SGIS.texture_color_mask to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension implements the same functionality for texture
	updates that glColorMask implements for color buffer updates.
	Masks for updating textures with indexed internal formats
	(the analog for glIndexMask) should be supported by a separate extension.
	
	The extension allows an application to update a subset of
	components in an existing texture.	The masks are applied after
	all pixel transfer operations have been performed, immediately
	prior to writing the texel value into texture memory.  They
	apply to all texture updates.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIS/texture_color_mask.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIS.texture_color_mask import *
from OpenGL.raw.GL.SGIS.texture_color_mask import _EXTENSION_NAME

def glInitTextureColorMaskSGIS():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Array-size annotations for OpenGL.raw.GLE

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw import GLE as raw

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL.raw.GL import _types as GL_types
GLvoid = GL_types.GLvoid
STRING = c_char_p


gleExtrusion = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            arrays.setInputArraySizeType(
                arrays.setInputArraySizeType(
                    raw.gleExtrusion,
                    None, # XXX Could not determine size of argument contour for gleExtrusion arrays.GLdoubleArray
                    arrays.GLdoubleArray, 
                    'contour',
                ),
                None, # XXX Could not determine size of argument cont_normal for gleExtrusion arrays.GLdoubleArray
                arrays.GLdoubleArray, 
                'cont_normal',
            ),
            None, # XXX Could not determine size of argument up for gleExtrusion arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'up',
        ),
        None, # XXX Could not determine size of argument point_array for gleExtrusion arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'point_array',
    ),
    None, # XXX Could not determine size of argument color_array for gleExtrusion arrays.GLfloatArray
    arrays.GLfloatArray, 
    'color_array',
)

gleHelicoid = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        raw.gleHelicoid,
        None, # XXX Could not determine size of argument startXform for gleHelicoid arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'startXform',
    ),
    None, # XXX Could not determine size of argument dXformdTheta for gleHelicoid arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'dXformdTheta',
)

gleLathe = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            arrays.setInputArraySizeType(
                arrays.setInputArraySizeType(
                    raw.gleLathe,
                    None, # XXX Could not determine size of argument contour for gleLathe arrays.GLdoubleArray
                    arrays.GLdoubleArray, 
                    'contour',
                ),
                None, # XXX Could not determine size of argument cont_normal for gleLathe arrays.GLdoubleArray
                arrays.GLdoubleArray, 
                'cont_normal',
            ),
            None, # XXX Could not determine size of argument up for gleLathe arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'up',
        ),
        None, # XXX Could not determine size of argument startXform for gleLathe arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'startXform',
    ),
    None, # XXX Could not determine size of argument dXformdTheta for gleLathe arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'dXformdTheta',
)

glePolyCone = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            raw.glePolyCone,
            None, # XXX Could not determine size of argument point_array for glePolyCone arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'point_array',
        ),
        None, # XXX Could not determine size of argument color_array for glePolyCone arrays.GLfloatArray
        arrays.GLfloatArray, 
        'color_array',
    ),
    None, # XXX Could not determine size of argument radius_array for glePolyCone arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'radius_array',
)

glePolyCylinder = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        raw.glePolyCylinder,
        None, # XXX Could not determine size of argument point_array for glePolyCylinder arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'point_array',
    ),
    None, # XXX Could not determine size of argument color_array for glePolyCylinder arrays.GLfloatArray
    arrays.GLfloatArray, 
    'color_array',
)

gleScrew = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            raw.gleScrew,
            None, # XXX Could not determine size of argument contour for gleScrew arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'contour',
        ),
        None, # XXX Could not determine size of argument cont_normal for gleScrew arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'cont_normal',
    ),
    None, # XXX Could not determine size of argument up for gleScrew arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'up',
)

gleSpiral = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            arrays.setInputArraySizeType(
                arrays.setInputArraySizeType(
                    raw.gleSpiral,
                    None, # XXX Could not determine size of argument contour for gleSpiral arrays.GLdoubleArray
                    arrays.GLdoubleArray, 
                    'contour',
                ),
                None, # XXX Could not determine size of argument cont_normal for gleSpiral arrays.GLdoubleArray
                arrays.GLdoubleArray, 
                'cont_normal',
            ),
            None, # XXX Could not determine size of argument up for gleSpiral arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'up',
        ),
        None, # XXX Could not determine size of argument startXform for gleSpiral arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'startXform',
    ),
    None, # XXX Could not determine size of argument dXformdTheta for gleSpiral arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'dXformdTheta',
)

gleSuperExtrusion = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            arrays.setInputArraySizeType(
                arrays.setInputArraySizeType(
                    arrays.setInputArraySizeType(
                        raw.gleSuperExtrusion,
                        None, # XXX Could not determine size of argument contour for gleSuperExtrusion arrays.GLdoubleArray
                        arrays.GLdoubleArray, 
                        'contour',
                    ),
                    None, # XXX Could not determine size of argument cont_normal for gleSuperExtrusion arrays.GLdoubleArray
                    arrays.GLdoubleArray, 
                    'cont_normal',
                ),
                None, # XXX Could not determine size of argument up for gleSuperExtrusion arrays.GLdoubleArray
                arrays.GLdoubleArray, 
                'up',
            ),
            None, # XXX Could not determine size of argument point_array for gleSuperExtrusion arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'point_array',
        ),
        None, # XXX Could not determine size of argument color_array for gleSuperExtrusion arrays.GLfloatArray
        arrays.GLfloatArray, 
        'color_array',
    ),
    None, # XXX Could not determine size of argument xform_array for gleSuperExtrusion arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'xform_array',
)

gleToroid = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        raw.gleToroid,
        None, # XXX Could not determine size of argument startXform for gleToroid arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'startXform',
    ),
    None, # XXX Could not determine size of argument dXformdTheta for gleToroid arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'dXformdTheta',
)

gleTwistExtrusion = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            arrays.setInputArraySizeType(
                arrays.setInputArraySizeType(
                    arrays.setInputArraySizeType(
                        raw.gleTwistExtrusion,
                        None, # XXX Could not determine size of argument contour for gleTwistExtrusion arrays.GLdoubleArray
                        arrays.GLdoubleArray, 
                        'contour',
                    ),
                    None, # XXX Could not determine size of argument cont_normal for gleTwistExtrusion arrays.GLdoubleArray
                    arrays.GLdoubleArray, 
                    'cont_normal',
                ),
                None, # XXX Could not determine size of argument up for gleTwistExtrusion arrays.GLdoubleArray
                arrays.GLdoubleArray, 
                'up',
            ),
            None, # XXX Could not determine size of argument point_array for gleTwistExtrusion arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'point_array',
        ),
        None, # XXX Could not determine size of argument color_array for gleTwistExtrusion arrays.GLfloatArray
        arrays.GLfloatArray, 
        'color_array',
    ),
    None, # XXX Could not determine size of argument twist_array for gleTwistExtrusion arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'twist_array',
)

rot_about_axis = arrays.setInputArraySizeType(
    raw.rot_about_axis,
    None, # XXX Could not determine size of argument axis for rot_about_axis arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'axis',
)

rot_axis = arrays.setInputArraySizeType(
    raw.rot_axis,
    None, # XXX Could not determine size of argument axis for rot_axis arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'axis',
)

rot_omega = arrays.setInputArraySizeType(
    raw.rot_omega,
    None, # XXX Could not determine size of argument axis for rot_omega arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'axis',
)

urot_about_axis = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        raw.urot_about_axis,
        None, # XXX Could not determine size of argument m for urot_about_axis arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'm',
    ),
    None, # XXX Could not determine size of argument axis for urot_about_axis arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'axis',
)

urot_axis = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        raw.urot_axis,
        None, # XXX Could not determine size of argument m for urot_axis arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'm',
    ),
    None, # XXX Could not determine size of argument axis for urot_axis arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'axis',
)

urot_omega = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        raw.urot_omega,
        None, # XXX Could not determine size of argument m for urot_omega arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'm',
    ),
    None, # XXX Could not determine size of argument axis for urot_omega arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'axis',
)

urot_prince = arrays.setInputArraySizeType(
    raw.urot_prince,
    None, # XXX Could not determine size of argument m for urot_prince arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'm',
)

uview_direction = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            raw.uview_direction,
            None, # XXX Could not determine size of argument m for uview_direction arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'm',
        ),
        None, # XXX Could not determine size of argument v21 for uview_direction arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'v21',
    ),
    None, # XXX Could not determine size of argument up for uview_direction arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'up',
)

uviewpoint = arrays.setInputArraySizeType(
    arrays.setInputArraySizeType(
        arrays.setInputArraySizeType(
            arrays.setInputArraySizeType(
                raw.uviewpoint,
                None, # XXX Could not determine size of argument m for uviewpoint arrays.GLdoubleArray
                arrays.GLdoubleArray, 
                'm',
            ),
            None, # XXX Could not determine size of argument v1 for uviewpoint arrays.GLdoubleArray
            arrays.GLdoubleArray, 
            'v1',
        ),
        None, # XXX Could not determine size of argument v2 for uviewpoint arrays.GLdoubleArray
        arrays.GLdoubleArray, 
        'v2',
    ),
    None, # XXX Could not determine size of argument up for uviewpoint arrays.GLdoubleArray
    arrays.GLdoubleArray, 
    'up',
)

__all__ = [
    'gleExtrusion',
    'gleHelicoid',
    'gleLathe',
    'glePolyCone',
    'glePolyCylinder',
    'gleScrew',
    'gleSpiral',
    'gleSuperExtrusion',
    'gleToroid',
    'gleTwistExtrusion',
    'rot_about_axis',
    'rot_axis',
    'rot_omega',
    'urot_about_axis',
    'urot_axis',
    'urot_omega',
    'urot_prince',
    'uview_direction',
    'uviewpoint'
]



'''OpenGL extension EXT.texture_env_dot3

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.texture_env_dot3 to provide a more 
Python-friendly API

Overview (from the spec)
	
	Adds new operation to the texture combiner operations.
	
	    DOT3_RGB_EXT                    Arg0 <dotprod> Arg1
	    DOT3_RGBA_EXT                   Arg0 <dotprod> Arg1
	
	where Arg0, Arg1 are derived from
	
	    PRIMARY_COLOR_EXT       primary color of incoming fragment
	    TEXTURE                 texture color of corresponding texture unit
	    CONSTANT_EXT            texture environment constant color
	    PREVIOUS_EXT            result of previous texture environment; on
	                            texture unit 0, this maps to PRIMARY_COLOR_EXT
	
	This operaion can only be performed if SOURCE0_RGB_EXT,
	SOURCE1_RGB_EXT are defined.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_env_dot3.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.texture_env_dot3 import *
from OpenGL.raw.GL.EXT.texture_env_dot3 import _EXTENSION_NAME

def glInitTextureEnvDot3EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIS_point_line_texgen'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIS_point_line_texgen',error_checker=_errors._error_checker)
GL_EYE_DISTANCE_TO_LINE_SGIS=_C('GL_EYE_DISTANCE_TO_LINE_SGIS',0x81F2)
GL_EYE_DISTANCE_TO_POINT_SGIS=_C('GL_EYE_DISTANCE_TO_POINT_SGIS',0x81F0)
GL_EYE_LINE_SGIS=_C('GL_EYE_LINE_SGIS',0x81F6)
GL_EYE_POINT_SGIS=_C('GL_EYE_POINT_SGIS',0x81F4)
GL_OBJECT_DISTANCE_TO_LINE_SGIS=_C('GL_OBJECT_DISTANCE_TO_LINE_SGIS',0x81F3)
GL_OBJECT_DISTANCE_TO_POINT_SGIS=_C('GL_OBJECT_DISTANCE_TO_POINT_SGIS',0x81F1)
GL_OBJECT_LINE_SGIS=_C('GL_OBJECT_LINE_SGIS',0x81F7)
GL_OBJECT_POINT_SGIS=_C('GL_OBJECT_POINT_SGIS',0x81F5)




'''OpenGL extension ARB.fragment_layer_viewport

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.fragment_layer_viewport to provide a more 
Python-friendly API

Overview (from the spec)
	
	The geometry shader has the special built-in variables gl_Layer and
	gl_ViewportIndex that specify which layer and viewport primitives
	are rendered to. Currently the fragment shader does not know which
	layer or viewport the fragments are being written to without the
	application implementing their own interface variables between
	the geometry and fragment shaders.
	
	This extension specifies that the gl_Layer and gl_ViewportIndex
	built-in variables are also available to the fragment shader so the
	application doesn't need to implement these manually.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/fragment_layer_viewport.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.fragment_layer_viewport import *
from OpenGL.raw.GL.ARB.fragment_layer_viewport import _EXTENSION_NAME

def glInitFragmentLayerViewportARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""C-style "raw" API for low-level ctypes-specific access to OpenGL

This sub-package is autogenerated using a customised version
of the ctypes codegenerator package (see src/openglgenerator.py and
src/generateraw.py).
"""


'''OpenGL extension EXT.rescale_normal

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.rescale_normal to provide a more 
Python-friendly API

Overview (from the spec)
	
	When normal rescaling is enabled a new operation is added to the
	transformation of the normal vector into eye coordinates.  The normal vector 
	is rescaled after it is multiplied by the inverse modelview matrix and 
	before it is normalized.  
	
	The rescale factor is chosen so that in many cases normal vectors with unit
	length in object coordinates will not need to be normalized as they
	are transformed into eye coordinates.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/rescale_normal.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.rescale_normal import *
from OpenGL.raw.GL.EXT.rescale_normal import _EXTENSION_NAME

def glInitRescaleNormalEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.gpu_program5_mem_extended

This module customises the behaviour of the 
OpenGL.raw.GL.NV.gpu_program5_mem_extended to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new set of storage modifiers that can be used by
	NV_gpu_program5 assembly program instructions loading from or storing to
	various forms of GPU memory.  In particular, we provide support for loads
	and stores using the storage modifiers:
	
	    .F16X2  .F16X4  .F16    (for 16-bit floating-point scalars/vectors)
	    .S8X2   .S8X4           (for 8-bit signed integer vectors)
	    .S16X2  .S16X4          (for 16-bit signed integer vectors)
	    .U8X2   .U8X4           (for 8-bit unsigned integer vectors)
	    .U16X2  .U16X4          (for 16-bit unsigned integer vectors)
	
	These modifiers are allowed for the following load/store instructions:
	
	    LDC             Load from constant buffer
	
	    LOAD            Global load
	    STORE           Global store
	
	    LOADIM          Image load (via EXT_shader_image_load_store)
	    STOREIM         Image store (via EXT_shader_image_load_store)
	
	    LDB             Load from storage buffer (via 
	                      NV_shader_storage_buffer_object) 
	    STB             Store to storage buffer (via 
	                      NV_shader_storage_buffer_object) 
	
	    LDS             Load from shared memory (via NV_compute_program5)
	    STS             Store to shared memory (via NV_compute_program5)
	
	For assembly programs prior to this extension, it was necessary to access
	memory using packed types and then unpack with additional shader
	instructions.
	
	Similar capabilities have already been provided in the OpenGL Shading
	Language (GLSL) via the NV_gpu_shader5 extension, using the extended data
	types provided there (e.g., "float16_t", "u8vec4", "s16vec2").

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/gpu_program5_mem_extended.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.gpu_program5_mem_extended import *
from OpenGL.raw.GL.NV.gpu_program5_mem_extended import _EXTENSION_NAME

def glInitGpuProgram5MemExtendedNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_blend_func_separate'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_blend_func_separate',error_checker=_errors._error_checker)
GL_BLEND_DST_ALPHA_EXT=_C('GL_BLEND_DST_ALPHA_EXT',0x80CA)
GL_BLEND_DST_RGB_EXT=_C('GL_BLEND_DST_RGB_EXT',0x80C8)
GL_BLEND_SRC_ALPHA_EXT=_C('GL_BLEND_SRC_ALPHA_EXT',0x80CB)
GL_BLEND_SRC_RGB_EXT=_C('GL_BLEND_SRC_RGB_EXT',0x80C9)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glBlendFuncSeparateEXT(sfactorRGB,dfactorRGB,sfactorAlpha,dfactorAlpha):pass



'''OpenGL extension ARB.get_proc_address

This module customises the behaviour of the 
OpenGL.raw.GLX.ARB.get_proc_address to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a function to return the address of GLX
	and GL extension functions, given the function name. This is
	necessary with (for example) heterogenous implementations where
	hardware drivers may implement extension functions not known to the
	link library; a similar situation on Windows implementations
	resulted in the wglGetProcAddress function.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/get_proc_address.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.ARB.get_proc_address import *
from OpenGL.raw.GLX.ARB.get_proc_address import _EXTENSION_NAME

def glInitGetProcAddressARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_ES2_compatibility'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_ES2_compatibility',error_checker=_errors._error_checker)
GL_FIXED=_C('GL_FIXED',0x140C)
GL_HIGH_FLOAT=_C('GL_HIGH_FLOAT',0x8DF2)
GL_HIGH_INT=_C('GL_HIGH_INT',0x8DF5)
GL_IMPLEMENTATION_COLOR_READ_FORMAT=_C('GL_IMPLEMENTATION_COLOR_READ_FORMAT',0x8B9B)
GL_IMPLEMENTATION_COLOR_READ_TYPE=_C('GL_IMPLEMENTATION_COLOR_READ_TYPE',0x8B9A)
GL_LOW_FLOAT=_C('GL_LOW_FLOAT',0x8DF0)
GL_LOW_INT=_C('GL_LOW_INT',0x8DF3)
GL_MAX_FRAGMENT_UNIFORM_VECTORS=_C('GL_MAX_FRAGMENT_UNIFORM_VECTORS',0x8DFD)
GL_MAX_VARYING_VECTORS=_C('GL_MAX_VARYING_VECTORS',0x8DFC)
GL_MAX_VERTEX_UNIFORM_VECTORS=_C('GL_MAX_VERTEX_UNIFORM_VECTORS',0x8DFB)
GL_MEDIUM_FLOAT=_C('GL_MEDIUM_FLOAT',0x8DF1)
GL_MEDIUM_INT=_C('GL_MEDIUM_INT',0x8DF4)
GL_NUM_SHADER_BINARY_FORMATS=_C('GL_NUM_SHADER_BINARY_FORMATS',0x8DF9)
GL_RGB565=_C('GL_RGB565',0x8D62)
GL_SHADER_BINARY_FORMATS=_C('GL_SHADER_BINARY_FORMATS',0x8DF8)
GL_SHADER_COMPILER=_C('GL_SHADER_COMPILER',0x8DFA)
@_f
@_p.types(None,_cs.GLfloat)
def glClearDepthf(d):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glDepthRangef(n,f):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray,arrays.GLintArray)
def glGetShaderPrecisionFormat(shadertype,precisiontype,range,precision):pass
@_f
@_p.types(None,)
def glReleaseShaderCompiler():pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei)
def glShaderBinary(count,shaders,binaryformat,binary,length):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_AMD_performance_monitor'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_AMD_performance_monitor',error_checker=_errors._error_checker)
GL_COUNTER_RANGE_AMD=_C('GL_COUNTER_RANGE_AMD',0x8BC1)
GL_COUNTER_TYPE_AMD=_C('GL_COUNTER_TYPE_AMD',0x8BC0)
GL_PERCENTAGE_AMD=_C('GL_PERCENTAGE_AMD',0x8BC3)
GL_PERFMON_RESULT_AMD=_C('GL_PERFMON_RESULT_AMD',0x8BC6)
GL_PERFMON_RESULT_AVAILABLE_AMD=_C('GL_PERFMON_RESULT_AVAILABLE_AMD',0x8BC4)
GL_PERFMON_RESULT_SIZE_AMD=_C('GL_PERFMON_RESULT_SIZE_AMD',0x8BC5)
GL_UNSIGNED_INT64_AMD=_C('GL_UNSIGNED_INT64_AMD',0x8BC2)
@_f
@_p.types(None,_cs.GLuint)
def glBeginPerfMonitorAMD(monitor):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeletePerfMonitorsAMD(n,monitors):pass
@_f
@_p.types(None,_cs.GLuint)
def glEndPerfMonitorAMD(monitor):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenPerfMonitorsAMD(n,monitors):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray,arrays.GLintArray)
def glGetPerfMonitorCounterDataAMD(monitor,pname,dataSize,data,bytesWritten):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,ctypes.c_void_p)
def glGetPerfMonitorCounterInfoAMD(group,counter,pname,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetPerfMonitorCounterStringAMD(group,counter,bufSize,length,counterString):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLintArray,arrays.GLintArray,_cs.GLsizei,arrays.GLuintArray)
def glGetPerfMonitorCountersAMD(group,numCounters,maxActiveCounters,counterSize,counters):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetPerfMonitorGroupStringAMD(group,bufSize,length,groupString):pass
@_f
@_p.types(None,arrays.GLintArray,_cs.GLsizei,arrays.GLuintArray)
def glGetPerfMonitorGroupsAMD(numGroups,groupsSize,groups):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLboolean,_cs.GLuint,_cs.GLint,arrays.GLuintArray)
def glSelectPerfMonitorCountersAMD(monitor,enable,group,numCounters,counterList):pass



'''OpenGL extension ARB.texture_compression_rgtc

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_compression_rgtc to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces four new block-based texture compression
	formats suited for unsigned and signed red and red-green textures
	(hence the name "rgtc" for Red-Green Texture Compression).
	
	These formats are designed to reduce the storage requirements
	and memory bandwidth required for red and red-green textures by
	a factor of 2-to-1 over conventional uncompressed luminance and
	luminance-alpha textures with 8-bit components (GL_LUMINANCE8 and
	GL_LUMINANCE8_ALPHA8).
	
	The compressed signed red-green format is reasonably suited for
	storing compressed normal maps.
	
	This extension uses the same compression format as the
	EXT_texture_compression_latc extension except the color data is stored
	in the red and green components rather than luminance and alpha.
	Representing compressed red and green components is consistent with
	the BC4 and BC5 compressed formats supported by DirectX 10.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_compression_rgtc.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_compression_rgtc import *
from OpenGL.raw.GL.ARB.texture_compression_rgtc import _EXTENSION_NAME

def glInitTextureCompressionRgtcARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_pixel_data_range'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_pixel_data_range',error_checker=_errors._error_checker)
GL_READ_PIXEL_DATA_RANGE_LENGTH_NV=_C('GL_READ_PIXEL_DATA_RANGE_LENGTH_NV',0x887B)
GL_READ_PIXEL_DATA_RANGE_NV=_C('GL_READ_PIXEL_DATA_RANGE_NV',0x8879)
GL_READ_PIXEL_DATA_RANGE_POINTER_NV=_C('GL_READ_PIXEL_DATA_RANGE_POINTER_NV',0x887D)
GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV=_C('GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV',0x887A)
GL_WRITE_PIXEL_DATA_RANGE_NV=_C('GL_WRITE_PIXEL_DATA_RANGE_NV',0x8878)
GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV=_C('GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV',0x887C)
@_f
@_p.types(None,_cs.GLenum)
def glFlushPixelDataRangeNV(target):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glPixelDataRangeNV(target,length,pointer):pass



'''OpenGL extension AMD.texture_texture4

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.texture_texture4 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds new shading language built-in texture functions
	to the shading language.
	
	These texture functions may be used to access one component textures.
	
	The texture4 built-in function returns a texture value derived from
	a 2x2 set of texels in the image array of level levelbase is selected.
	These texels are selected in the same way as when the value of
	TEXTURE_MIN_FILTER is LINEAR, but instead of these texels being
	filtered to generate the texture value, the R, G, B and A texture values
	are derived directly from these four texels.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/texture_texture4.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.texture_texture4 import *
from OpenGL.raw.GL.AMD.texture_texture4 import _EXTENSION_NAME

def glInitTextureTexture4AMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.multisample

This module customises the behaviour of the 
OpenGL.raw.GLX.ARB.multisample to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a mechanism to antialias all GL primitives:
	points, lines, polygons, bitmaps, and images.  The technique is to
	sample all primitives multiple times at each pixel.  The color
	sample values are resolved to a single, displayable color each time
	a pixel is updated, so the antialiasing appears to be automatic at
	the application level.  Because each sample includes depth and
	stencil information, the depth and stencil functions perform
	equivalently to the single-sample mode.
	
	An additional buffer, called the multisample buffer, is added to
	the framebuffer.  Pixel sample values, including color, depth, and
	stencil values, are stored in this buffer.  When the framebuffer
	includes a multisample buffer, it does not also include separate
	depth or stencil buffers, even if the multisample buffer does not
	store depth or stencil values.  Color buffers (left/right, front/
	back, and aux) do coexist with the multisample buffer, however.
	
	Multisample antialiasing is most valuable for rendering polygons,
	because it requires no sorting for hidden surface elimination, and
	it correctly handles adjacent polygons, object silhouettes, and
	even intersecting polygons.  If only points or lines are being
	rendered, the "smooth" antialiasing mechanism provided by the base
	GL may result in a higher quality image.  This extension is
	designed to allow multisample and smooth antialiasing techniques
	to be alternated during the rendering of a single scene.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/multisample.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.ARB.multisample import *
from OpenGL.raw.GLX.ARB.multisample import _EXTENSION_NAME

def glInitMultisampleARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.draw_buffers_blend

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.draw_buffers_blend to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds upon the ARB_draw_buffers and
	EXT_draw_buffers2 extensions. In ARB_draw_buffers (part of OpenGL
	2.0), separate values could be written to each color buffer. This
	was further enhanced by EXT_draw_buffers2 by adding in the ability
	to enable blending and to set color write masks independently per
	color output.
	
	This extension provides the ability to set individual blend
	equations and blend functions for each color output.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/draw_buffers_blend.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.draw_buffers_blend import *
from OpenGL.raw.GL.ARB.draw_buffers_blend import _EXTENSION_NAME

def glInitDrawBuffersBlendARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.uniform_buffer_object

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.uniform_buffer_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces the concept of a group of GLSL uniforms
	known as a "uniform block", and the API mechanisms to store "uniform
	blocks" in GL buffer objects.
	
	The extension also defines both a standard cross-platform layout in
	memory for uniform block data, as well as mechanisms to allow the GL
	to optimize the data layout in an implementation-defined manner.
	
	Prior to this extension, the existing interface for modification of
	uniform values allowed modification of large numbers of values using
	glUniform* calls, but only for a single uniform name (or a uniform
	array) at a time. However, updating uniforms in this manner may not
	map well to heterogenous uniform data structures defined for a GL
	application and in these cases, the application is forced to either:
	
	    A) restructure their uniform data definitions into arrays
	    or
	    B) make an excessive number of calls through the GL interface
	       to one of the Uniform* variants.
	
	These solutions have their disadvantages. Solution A imposes
	considerable development overhead on the application developer. 
	Solution B may impose considerable run-time overhead on the
	application if the number of uniforms modified in a given frame of
	rendering is sufficiently large.
	
	This extension provides a better alternative to either (A) or (B) by
	allowing buffer object backing for the storage associated with all
	uniforms of a given GLSL program.
	
	Storing uniform blocks in buffer objects enables several key use
	cases:
	
	 - sharing of uniform data storage between program objects and
	   between program stages
	
	 - rapid swapping of sets of previously defined uniforms by storing
	   sets of uniform data on the GL server
	
	 - rapid updates of uniform data from both the client and the server
	
	The data storage for a uniform block can be declared to use one of
	three layouts in memory: packed, shared, or std140.
	
	  - "packed" uniform blocks have an implementation-dependent data
	    layout for efficiency, and unused uniforms may be eliminated by
	    the compiler to save space.
	
	  - "shared" uniform blocks, the default layout, have an implementation- 
	    dependent data layout for efficiency, but the layout will be uniquely
	    determined by the structure of the block, allowing data storage to be
	    shared across programs.
	
	  - "std140" uniform blocks have a standard cross-platform cross-vendor
	    layout (see below). Unused uniforms will not be eliminated.
	
	Any uniforms not declared in a named uniform block are said to 
	be part of the "default uniform block".
	
	While uniforms in the default uniform block are updated with
	glUniform* entry points and can have static initializers, uniforms
	in named uniform blocks are not. Instead, uniform block data is updated
	using the routines that update buffer objects and can not use static
	initializers.
	
	Rules and Concepts Guiding this Specification:
	
	For reference, a uniform has a "uniform index" (subsequently
	referred to as "u_index) and also a "uniform location" to
	efficiently identify it in the uniform data store of the
	implementation. We subsequently refer to this uniform data store of
	the implementation as the "uniform database".
	
	A "uniform block" only has a "uniform block index" used for queries
	and connecting the "uniform block" to a buffer object. A "uniform
	block" has no "location" because "uniform blocks" are not updated
	directly. The buffer object APIs are used instead.
	
	Properties of Uniforms and uniform blocks:
	
	a) A uniform is "active" if it exists in the database and has a valid
	   u_index.
	b) A "uniform block" is "active" if it exists in the database and
	   has a valid ub_index.
	c) Uniforms and "uniform blocks" can be inactive because they don't
	   exist in the source, or because they have been removed by dead
	   code elimination.
	d) An inactive uniform has u_index == INVALID_INDEX.
	e) An inactive uniform block has ub_index == INVALID_INDEX.
	f) A u_index or ub_index of INVALID_INDEX generates the
	   INVALID_VALUE error if given as a function argument.
	g) The default uniform block, which is not assigned any ub_index, uses a
	   private, internal data storage, and does not have any buffer object
	   associated with it.
	h) An active uniform that is a member of the default uniform block has
	   location >= 0 and it has offset == stride == -1.
	i) An active uniform that is a member of a named uniform block has
	   location == -1.
	j) A uniform location of -1 is silently ignored if given as a function
	   argument.
	k) Uniform block declarations may not be nested

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/uniform_buffer_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.uniform_buffer_object import *
from OpenGL.raw.GL.ARB.uniform_buffer_object import _EXTENSION_NAME

def glInitUniformBufferObjectARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetUniformIndices.uniformNames size not checked against 'uniformCount'
glGetUniformIndices=wrapper.wrapper(glGetUniformIndices).setOutput(
    'uniformIndices',size=_glgets._glget_size_mapping,pnameArg='uniformCount',orPassIn=True
).setInputArraySize(
    'uniformNames', None
)
# OUTPUT glGetActiveUniformsiv.params COMPSIZE(uniformCount, pname) 
# INPUT glGetActiveUniformsiv.uniformIndices size not checked against uniformCount
glGetActiveUniformsiv=wrapper.wrapper(glGetActiveUniformsiv).setInputArraySize(
    'uniformIndices', None
)
glGetActiveUniformName=wrapper.wrapper(glGetActiveUniformName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'uniformName',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
# INPUT glGetUniformBlockIndex.uniformBlockName size not checked against ''
glGetUniformBlockIndex=wrapper.wrapper(glGetUniformBlockIndex).setInputArraySize(
    'uniformBlockName', None
)
# OUTPUT glGetActiveUniformBlockiv.params COMPSIZE(program, uniformBlockIndex, pname) 
glGetActiveUniformBlockName=wrapper.wrapper(glGetActiveUniformBlockName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'uniformBlockName',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_sRGB_decode

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.texture_sRGB_decode to provide a more 
Python-friendly API

Overview (from the spec)
	
	The EXT_texture_sRGB extension (promoted to core in OpenGL 2.1)
	provides a texture format stored in the sRGB color space. Sampling one
	of these textures will always return the color value decoded into a
	linear color space. However, an application may wish to sample and
	retrieve the undecoded sRGB data from the texture and manipulate
	that directly.
	
	This extension adds a Texture Parameter and Sampler Object parameter to
	allow sRGB textures to be read directly, without decoding.
	
	The new parameter, TEXTURE_SRGB_DECODE_EXT controls whether the 
	decoding happens at sample time. It only applies to textures with an 
	internal format that is sRGB and is ignored for all other textures. 
	This value defaults to DECODE_EXT, which indicates the texture 
	should be decoded to linear color space.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_sRGB_decode.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.texture_sRGB_decode import *
from OpenGL.raw.GL.EXT.texture_sRGB_decode import _EXTENSION_NAME

def glInitTextureSrgbDecodeEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.draw_buffers

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.draw_buffers to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension extends ARB_fragment_program and ARB_fragment_shader
	to allow multiple output colors, and provides a mechanism for
	directing those outputs to multiple color buffers.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/draw_buffers.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.draw_buffers import *
from OpenGL.raw.GL.ARB.draw_buffers import _EXTENSION_NAME

def glInitDrawBuffersARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawBuffersARB.bufs size not checked against n
glDrawBuffersARB=wrapper.wrapper(glDrawBuffersARB).setInputArraySize(
    'bufs', None
)
### END AUTOGENERATED SECTION
from OpenGL.lazywrapper import lazy as _lazy
@_lazy( glDrawBuffersARB )
def glDrawBuffersARB( baseOperation, n=None, bufs=None ):
    """glDrawBuffersARB( bufs ) -> bufs 
    
    Wrapper will calculate n from dims of bufs if only 
    one argument is provided...
    """
    if bufs is None:
        bufs = n
        n = None
    bufs = arrays.GLenumArray.asArray( bufs )
    if n is None:
        n = arrays.GLenumArray.arraySize( bufs )
    return baseOperation( n,bufs )



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_QCOM_tiled_rendering'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_QCOM_tiled_rendering',error_checker=_errors._error_checker)
GL_COLOR_BUFFER_BIT0_QCOM=_C('GL_COLOR_BUFFER_BIT0_QCOM',0x00000001)
GL_COLOR_BUFFER_BIT1_QCOM=_C('GL_COLOR_BUFFER_BIT1_QCOM',0x00000002)
GL_COLOR_BUFFER_BIT2_QCOM=_C('GL_COLOR_BUFFER_BIT2_QCOM',0x00000004)
GL_COLOR_BUFFER_BIT3_QCOM=_C('GL_COLOR_BUFFER_BIT3_QCOM',0x00000008)
GL_COLOR_BUFFER_BIT4_QCOM=_C('GL_COLOR_BUFFER_BIT4_QCOM',0x00000010)
GL_COLOR_BUFFER_BIT5_QCOM=_C('GL_COLOR_BUFFER_BIT5_QCOM',0x00000020)
GL_COLOR_BUFFER_BIT6_QCOM=_C('GL_COLOR_BUFFER_BIT6_QCOM',0x00000040)
GL_COLOR_BUFFER_BIT7_QCOM=_C('GL_COLOR_BUFFER_BIT7_QCOM',0x00000080)
GL_DEPTH_BUFFER_BIT0_QCOM=_C('GL_DEPTH_BUFFER_BIT0_QCOM',0x00000100)
GL_DEPTH_BUFFER_BIT1_QCOM=_C('GL_DEPTH_BUFFER_BIT1_QCOM',0x00000200)
GL_DEPTH_BUFFER_BIT2_QCOM=_C('GL_DEPTH_BUFFER_BIT2_QCOM',0x00000400)
GL_DEPTH_BUFFER_BIT3_QCOM=_C('GL_DEPTH_BUFFER_BIT3_QCOM',0x00000800)
GL_DEPTH_BUFFER_BIT4_QCOM=_C('GL_DEPTH_BUFFER_BIT4_QCOM',0x00001000)
GL_DEPTH_BUFFER_BIT5_QCOM=_C('GL_DEPTH_BUFFER_BIT5_QCOM',0x00002000)
GL_DEPTH_BUFFER_BIT6_QCOM=_C('GL_DEPTH_BUFFER_BIT6_QCOM',0x00004000)
GL_DEPTH_BUFFER_BIT7_QCOM=_C('GL_DEPTH_BUFFER_BIT7_QCOM',0x00008000)
GL_MULTISAMPLE_BUFFER_BIT0_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT0_QCOM',0x01000000)
GL_MULTISAMPLE_BUFFER_BIT1_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT1_QCOM',0x02000000)
GL_MULTISAMPLE_BUFFER_BIT2_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT2_QCOM',0x04000000)
GL_MULTISAMPLE_BUFFER_BIT3_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT3_QCOM',0x08000000)
GL_MULTISAMPLE_BUFFER_BIT4_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT4_QCOM',0x10000000)
GL_MULTISAMPLE_BUFFER_BIT5_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT5_QCOM',0x20000000)
GL_MULTISAMPLE_BUFFER_BIT6_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT6_QCOM',0x40000000)
GL_MULTISAMPLE_BUFFER_BIT7_QCOM=_C('GL_MULTISAMPLE_BUFFER_BIT7_QCOM',0x80000000)
GL_STENCIL_BUFFER_BIT0_QCOM=_C('GL_STENCIL_BUFFER_BIT0_QCOM',0x00010000)
GL_STENCIL_BUFFER_BIT1_QCOM=_C('GL_STENCIL_BUFFER_BIT1_QCOM',0x00020000)
GL_STENCIL_BUFFER_BIT2_QCOM=_C('GL_STENCIL_BUFFER_BIT2_QCOM',0x00040000)
GL_STENCIL_BUFFER_BIT3_QCOM=_C('GL_STENCIL_BUFFER_BIT3_QCOM',0x00080000)
GL_STENCIL_BUFFER_BIT4_QCOM=_C('GL_STENCIL_BUFFER_BIT4_QCOM',0x00100000)
GL_STENCIL_BUFFER_BIT5_QCOM=_C('GL_STENCIL_BUFFER_BIT5_QCOM',0x00200000)
GL_STENCIL_BUFFER_BIT6_QCOM=_C('GL_STENCIL_BUFFER_BIT6_QCOM',0x00400000)
GL_STENCIL_BUFFER_BIT7_QCOM=_C('GL_STENCIL_BUFFER_BIT7_QCOM',0x00800000)
@_f
@_p.types(None,_cs.GLbitfield)
def glEndTilingQCOM(preserveMask):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLbitfield)
def glStartTilingQCOM(x,y,width,height,preserveMask):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_multiview_draw_buffers'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_multiview_draw_buffers',error_checker=_errors._error_checker)
GL_COLOR_ATTACHMENT_EXT=_C('GL_COLOR_ATTACHMENT_EXT',0x90F0)
GL_DRAW_BUFFER_EXT=_C('GL_DRAW_BUFFER_EXT',0x0C01)
GL_MAX_MULTIVIEW_BUFFERS_EXT=_C('GL_MAX_MULTIVIEW_BUFFERS_EXT',0x90F2)
GL_MULTIVIEW_EXT=_C('GL_MULTIVIEW_EXT',0x90F1)
GL_READ_BUFFER_EXT=_C('GL_READ_BUFFER_EXT',0x0C02)
@_f
@_p.types(None,_cs.GLint,arrays.GLuintArray,arrays.GLintArray)
def glDrawBuffersIndexedEXT(n,location,indices):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glGetIntegeri_vEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glReadBufferIndexedEXT(src,index):pass



'''OpenGL extension NV.vertex_attrib_integer_64bit

This module customises the behaviour of the 
OpenGL.raw.GL.NV.vertex_attrib_integer_64bit to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides support for specifying vertex attributes with
	64-bit integer components, analagous to the 64-bit floating point support
	added in EXT_vertex_attrib_64bit.
	
	Additionally, it provides the VertexAttribLFormatNV entry point to specify
	bindless vertex attribute arrays with 64-bit integer or floating-point
	components in conjunction with the NV_vertex_buffer_unified_memory
	extension.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/vertex_attrib_integer_64bit.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.vertex_attrib_integer_64bit import *
from OpenGL.raw.GL.NV.vertex_attrib_integer_64bit import _EXTENSION_NAME

def glInitVertexAttribInteger64BitNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexAttribL1i64vNV=wrapper.wrapper(glVertexAttribL1i64vNV).setInputArraySize(
    'v', 1
)
glVertexAttribL2i64vNV=wrapper.wrapper(glVertexAttribL2i64vNV).setInputArraySize(
    'v', 2
)
glVertexAttribL3i64vNV=wrapper.wrapper(glVertexAttribL3i64vNV).setInputArraySize(
    'v', 3
)
glVertexAttribL4i64vNV=wrapper.wrapper(glVertexAttribL4i64vNV).setInputArraySize(
    'v', 4
)
glVertexAttribL1ui64vNV=wrapper.wrapper(glVertexAttribL1ui64vNV).setInputArraySize(
    'v', 1
)
glVertexAttribL2ui64vNV=wrapper.wrapper(glVertexAttribL2ui64vNV).setInputArraySize(
    'v', 2
)
glVertexAttribL3ui64vNV=wrapper.wrapper(glVertexAttribL3ui64vNV).setInputArraySize(
    'v', 3
)
glVertexAttribL4ui64vNV=wrapper.wrapper(glVertexAttribL4ui64vNV).setInputArraySize(
    'v', 4
)
glGetVertexAttribLi64vNV=wrapper.wrapper(glGetVertexAttribLi64vNV).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetVertexAttribLui64vNV=wrapper.wrapper(glGetVertexAttribLui64vNV).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.stencil_wrap

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.stencil_wrap to provide a more 
Python-friendly API

Overview (from the spec)
	
	Various algorithms use the stencil buffer to "count" the number of
	surfaces that a ray passes through.  As the ray passes into an object,
	the stencil buffer is incremented.  As the ray passes out of an object,
	the stencil buffer is decremented.
	
	GL requires that the stencil increment operation clamps to its maximum
	value.  For algorithms that depend on the difference between the sum
	of the increments and the sum of the decrements, clamping causes an
	erroneous result.
	
	This extension provides an enable for both maximum and minimum wrapping
	of stencil values.  Instead, the stencil value wraps in both directions.
	
	Two additional stencil operations are specified.  These new operations
	are similiar to the existing INCR and DECR operations, but they wrap
	their result instead of saturating it.  This functionality matches
	the new stencil operations introduced by DirectX 6.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/stencil_wrap.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.stencil_wrap import *
from OpenGL.raw.GL.EXT.stencil_wrap import _EXTENSION_NAME

def glInitStencilWrapEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension MESA.copy_sub_buffer

This module customises the behaviour of the 
OpenGL.raw.GLX.MESA.copy_sub_buffer to provide a more 
Python-friendly API

Overview (from the spec)
	
	The glxCopySubBufferMESA() function copies a rectangular region
	of the back color buffer to the front color buffer.  This can be
	used to quickly repaint 3D windows in response to expose events
	when the back color buffer cannot be damaged by other windows.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/copy_sub_buffer.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.MESA.copy_sub_buffer import *
from OpenGL.raw.GLX.MESA.copy_sub_buffer import _EXTENSION_NAME

def glInitCopySubBufferMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.fragment_program_option

This module customises the behaviour of the 
OpenGL.raw.GL.NV.fragment_program_option to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional fragment program functionality
	to extend the standard ARB_fragment_program language and execution
	environment.  ARB programs wishing to use this added functionality
	need only add:
	
	    OPTION NV_fragment_program;
	
	to the beginning of their fragment programs.
	
	The functionality provided by this extension, which is roughly
	equivalent to that provided by the NV_fragment_program extension,
	includes:
	
	  * increased control over precision in arithmetic computations and
	    storage,
	
	  * data-dependent conditional writemasks,
	
	  * an absolute value operator on scalar and swizzled operand loads,
	
	  * instructions to compute partial derivatives, and perform texture
	    lookups using specified partial derivatives,
	
	  * fully orthogonal "set on" instructions,
	
	  * instructions to compute reflection vector and perform a 2D
	    coordinate transform, and
	
	  * instructions to pack and unpack multiple quantities into a single
	    component.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/fragment_program_option.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.fragment_program_option import *
from OpenGL.raw.GL.NV.fragment_program_option import _EXTENSION_NAME

def glInitFragmentProgramOptionNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension INTEL.parallel_arrays

This module customises the behaviour of the 
OpenGL.raw.GL.INTEL.parallel_arrays to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the ability to format vertex arrays in a way that's 

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/INTEL/parallel_arrays.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.INTEL.parallel_arrays import *
from OpenGL.raw.GL.INTEL.parallel_arrays import _EXTENSION_NAME

def glInitParallelArraysINTEL():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexPointervINTEL=wrapper.wrapper(glVertexPointervINTEL).setInputArraySize(
    'pointer', 4
)
glNormalPointervINTEL=wrapper.wrapper(glNormalPointervINTEL).setInputArraySize(
    'pointer', 4
)
glColorPointervINTEL=wrapper.wrapper(glColorPointervINTEL).setInputArraySize(
    'pointer', 4
)
glTexCoordPointervINTEL=wrapper.wrapper(glTexCoordPointervINTEL).setInputArraySize(
    'pointer', 4
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_texture_snorm'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_texture_snorm',error_checker=_errors._error_checker)
GL_ALPHA16_SNORM=_C('GL_ALPHA16_SNORM',0x9018)
GL_ALPHA8_SNORM=_C('GL_ALPHA8_SNORM',0x9014)
GL_ALPHA_SNORM=_C('GL_ALPHA_SNORM',0x9010)
GL_INTENSITY16_SNORM=_C('GL_INTENSITY16_SNORM',0x901B)
GL_INTENSITY8_SNORM=_C('GL_INTENSITY8_SNORM',0x9017)
GL_INTENSITY_SNORM=_C('GL_INTENSITY_SNORM',0x9013)
GL_LUMINANCE16_ALPHA16_SNORM=_C('GL_LUMINANCE16_ALPHA16_SNORM',0x901A)
GL_LUMINANCE16_SNORM=_C('GL_LUMINANCE16_SNORM',0x9019)
GL_LUMINANCE8_ALPHA8_SNORM=_C('GL_LUMINANCE8_ALPHA8_SNORM',0x9016)
GL_LUMINANCE8_SNORM=_C('GL_LUMINANCE8_SNORM',0x9015)
GL_LUMINANCE_ALPHA_SNORM=_C('GL_LUMINANCE_ALPHA_SNORM',0x9012)
GL_LUMINANCE_SNORM=_C('GL_LUMINANCE_SNORM',0x9011)
GL_R16_SNORM=_C('GL_R16_SNORM',0x8F98)
GL_R8_SNORM=_C('GL_R8_SNORM',0x8F94)
GL_RED_SNORM=_C('GL_RED_SNORM',0x8F90)
GL_RG16_SNORM=_C('GL_RG16_SNORM',0x8F99)
GL_RG8_SNORM=_C('GL_RG8_SNORM',0x8F95)
GL_RGB16_SNORM=_C('GL_RGB16_SNORM',0x8F9A)
GL_RGB8_SNORM=_C('GL_RGB8_SNORM',0x8F96)
GL_RGBA16_SNORM=_C('GL_RGBA16_SNORM',0x8F9B)
GL_RGBA8_SNORM=_C('GL_RGBA8_SNORM',0x8F97)
GL_RGBA_SNORM=_C('GL_RGBA_SNORM',0x8F93)
GL_RGB_SNORM=_C('GL_RGB_SNORM',0x8F92)
GL_RG_SNORM=_C('GL_RG_SNORM',0x8F91)
GL_SIGNED_NORMALIZED=_C('GL_SIGNED_NORMALIZED',0x8F9C)




'''OpenGL extension SGIS.texture4D

This module customises the behaviour of the 
OpenGL.raw.GL.SGIS.texture4D to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines 4-dimensional texture mapping.  If EXT_subtexture
	is supported, this extension also defines a mechanism to redefine a
	portion of an existing 4-dimensional texture image.  Because
	EXT_texture3D is required, this extension utilizes the 3-dimensional
	image support defined in by EXT_texture3D as a base for 4-dimensional
	image manipulation.
	
	The 4th dimension has an abstract, rather than physical, reference
	and will be called "extent", since the definition of extent is "that which
	specifies the range or magnitude of an area or volume."
	
	Four-dimensional texture mapping is more constrained than its one, two,
	and three-dimensional counterparts.  Mipmapping is not supported, so
	only the level-zero 4-dimensional texture image can be defined.  Cubic
	filtering is not supported, so the border width must be either zero or
	one.
	
	Four-dimensional textures are used primarily as color lookup tables for
	color conversion.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIS/texture4D.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIS.texture4D import *
from OpenGL.raw.GL.SGIS.texture4D import _EXTENSION_NAME

def glInitTexture4DSGIS():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glTexImage4DSGIS.pixels size not checked against 'format,type,width,height,depth,size4d'
glTexImage4DSGIS=wrapper.wrapper(glTexImage4DSGIS).setInputArraySize(
    'pixels', None
)
# INPUT glTexSubImage4DSGIS.pixels size not checked against 'format,type,width,height,depth,size4d'
glTexSubImage4DSGIS=wrapper.wrapper(glTexSubImage4DSGIS).setInputArraySize(
    'pixels', None
)
### END AUTOGENERATED SECTION

from OpenGL.GL import images as _i
_i.images.RANK_PACKINGS.setdefault(4,[]).extend([
    (_i.GL_1_1.glPixelStorei,GL_PACK_SKIP_VOLUMES_SGIS, 0),
    (_i.GL_1_1.glPixelStorei,GL_PACK_IMAGE_DEPTH_SGIS, 0),
])



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_video_capture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_video_capture',error_checker=_errors._error_checker)
GL_FAILURE_NV=_C('GL_FAILURE_NV',0x9030)
GL_FIELD_LOWER_NV=_C('GL_FIELD_LOWER_NV',0x9023)
GL_FIELD_UPPER_NV=_C('GL_FIELD_UPPER_NV',0x9022)
GL_LAST_VIDEO_CAPTURE_STATUS_NV=_C('GL_LAST_VIDEO_CAPTURE_STATUS_NV',0x9027)
GL_NEXT_VIDEO_CAPTURE_BUFFER_STATUS_NV=_C('GL_NEXT_VIDEO_CAPTURE_BUFFER_STATUS_NV',0x9025)
GL_NUM_VIDEO_CAPTURE_STREAMS_NV=_C('GL_NUM_VIDEO_CAPTURE_STREAMS_NV',0x9024)
GL_PARTIAL_SUCCESS_NV=_C('GL_PARTIAL_SUCCESS_NV',0x902E)
GL_SUCCESS_NV=_C('GL_SUCCESS_NV',0x902F)
GL_VIDEO_BUFFER_BINDING_NV=_C('GL_VIDEO_BUFFER_BINDING_NV',0x9021)
GL_VIDEO_BUFFER_INTERNAL_FORMAT_NV=_C('GL_VIDEO_BUFFER_INTERNAL_FORMAT_NV',0x902D)
GL_VIDEO_BUFFER_NV=_C('GL_VIDEO_BUFFER_NV',0x9020)
GL_VIDEO_BUFFER_PITCH_NV=_C('GL_VIDEO_BUFFER_PITCH_NV',0x9028)
GL_VIDEO_CAPTURE_FIELD_LOWER_HEIGHT_NV=_C('GL_VIDEO_CAPTURE_FIELD_LOWER_HEIGHT_NV',0x903B)
GL_VIDEO_CAPTURE_FIELD_UPPER_HEIGHT_NV=_C('GL_VIDEO_CAPTURE_FIELD_UPPER_HEIGHT_NV',0x903A)
GL_VIDEO_CAPTURE_FRAME_HEIGHT_NV=_C('GL_VIDEO_CAPTURE_FRAME_HEIGHT_NV',0x9039)
GL_VIDEO_CAPTURE_FRAME_WIDTH_NV=_C('GL_VIDEO_CAPTURE_FRAME_WIDTH_NV',0x9038)
GL_VIDEO_CAPTURE_SURFACE_ORIGIN_NV=_C('GL_VIDEO_CAPTURE_SURFACE_ORIGIN_NV',0x903C)
GL_VIDEO_CAPTURE_TO_422_SUPPORTED_NV=_C('GL_VIDEO_CAPTURE_TO_422_SUPPORTED_NV',0x9026)
GL_VIDEO_COLOR_CONVERSION_MATRIX_NV=_C('GL_VIDEO_COLOR_CONVERSION_MATRIX_NV',0x9029)
GL_VIDEO_COLOR_CONVERSION_MAX_NV=_C('GL_VIDEO_COLOR_CONVERSION_MAX_NV',0x902A)
GL_VIDEO_COLOR_CONVERSION_MIN_NV=_C('GL_VIDEO_COLOR_CONVERSION_MIN_NV',0x902B)
GL_VIDEO_COLOR_CONVERSION_OFFSET_NV=_C('GL_VIDEO_COLOR_CONVERSION_OFFSET_NV',0x902C)
GL_YCBAYCR8A_4224_NV=_C('GL_YCBAYCR8A_4224_NV',0x9032)
GL_YCBYCR8_422_NV=_C('GL_YCBYCR8_422_NV',0x9031)
GL_Z4Y12Z4CB12Z4A12Z4Y12Z4CR12Z4A12_4224_NV=_C('GL_Z4Y12Z4CB12Z4A12Z4Y12Z4CR12Z4A12_4224_NV',0x9036)
GL_Z4Y12Z4CB12Z4CR12_444_NV=_C('GL_Z4Y12Z4CB12Z4CR12_444_NV',0x9037)
GL_Z4Y12Z4CB12Z4Y12Z4CR12_422_NV=_C('GL_Z4Y12Z4CB12Z4Y12Z4CR12_422_NV',0x9035)
GL_Z6Y10Z6CB10Z6A10Z6Y10Z6CR10Z6A10_4224_NV=_C('GL_Z6Y10Z6CB10Z6A10Z6Y10Z6CR10Z6A10_4224_NV',0x9034)
GL_Z6Y10Z6CB10Z6Y10Z6CR10_422_NV=_C('GL_Z6Y10Z6CB10Z6Y10Z6CR10_422_NV',0x9033)
@_f
@_p.types(None,_cs.GLuint)
def glBeginVideoCaptureNV(video_capture_slot):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,_cs.GLintptrARB)
def glBindVideoCaptureStreamBufferNV(video_capture_slot,stream,frame_region,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glBindVideoCaptureStreamTextureNV(video_capture_slot,stream,frame_region,target,texture):pass
@_f
@_p.types(None,_cs.GLuint)
def glEndVideoCaptureNV(video_capture_slot):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLdoubleArray)
def glGetVideoCaptureStreamdvNV(video_capture_slot,stream,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glGetVideoCaptureStreamfvNV(video_capture_slot,stream,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVideoCaptureStreamivNV(video_capture_slot,stream,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVideoCaptureivNV(video_capture_slot,pname,params):pass
@_f
@_p.types(_cs.GLenum,_cs.GLuint,arrays.GLuintArray,arrays.GLuint64Array)
def glVideoCaptureNV(video_capture_slot,sequence_num,capture_time):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLdoubleArray)
def glVideoCaptureStreamParameterdvNV(video_capture_slot,stream,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glVideoCaptureStreamParameterfvNV(video_capture_slot,stream,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glVideoCaptureStreamParameterivNV(video_capture_slot,stream,pname,params):pass



'''OpenGL extension NV.render_depth_texture

This module customises the behaviour of the 
OpenGL.raw.WGL.NV.render_depth_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows a depth buffer to be used for both rendering and
	texturing.  It is built upon the ARB_render_texture extension; the only
	addition in this extension is the ability to use a depth buffer as a
	DEPTH_COMPONENT texture map.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/render_depth_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.WGL import _types, _glgets
from OpenGL.raw.WGL.NV.render_depth_texture import *
from OpenGL.raw.WGL.NV.render_depth_texture import _EXTENSION_NAME

def glInitRenderDepthTextureNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.query_buffer_object

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.query_buffer_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	Statistics about the operation of the OpenGL pipeline, such as the number
	of samples that passed the depth test, the elapsed time between two events
	or the number of vertices written to a transform feedback buffer may
	be retrieved from the GL through query objects. The current value of a
	query object may be retrieved by the application through the OpenGL API.
	Should the result returned by the API be required for use in a shader,
	it must be passed back to the GL via a program uniform or some other
	mechanism. This requires a round-trip from the GPU to the CPU and back.
	
	This extension introduces a mechanism whereby the current value of a query
	result may be retrieved into a buffer object instead of client memory.
	This allows the query result to be made available to a shader without a
	round-trip to the CPU for example by subsequently using the buffer object
	as a uniform buffer, texture buffer or other data store visible to the
	shader. This functionality may also be used to place the results of
	many query objects into a single, large buffer and then map or otherwise
	read back the entire buffer at a later point in time, avoiding a per-query
	CPU-GPU synchronization event.
	
	The result of any query object type supported by the GL implementation
	may be retrieved into a buffer object. The implementation will determine
	the most efficient method of copying the query result to the buffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/query_buffer_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.query_buffer_object import *
from OpenGL.raw.GL.AMD.query_buffer_object import _EXTENSION_NAME

def glInitQueryBufferObjectAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Numbers passed as array handling code for PyOpenGL
"""
REGISTRY_NAME = 'numbers'
from OpenGL.raw.GL import _types 
from OpenGL.raw.GL.VERSION import GL_1_1
from OpenGL.arrays import formathandler
import ctypes
from OpenGL._bytes import long, integer_types

class NumberHandler( formathandler.FormatHandler ):
    """Allows the user to pass a bald Python float,int, etceteras as an array-of-1"""
    HANDLED_TYPES = integer_types + (
        float,
        _types.GLdouble,
        _types.GLfloat,
        _types.GLint,
        _types.GLshort,
        _types.GLuint,
        _types.GLulong,
        _types.GLushort,
        _types.GLclampf,
        _types.GLclampd,
    )
    def from_param( self, value, typeCode=None  ):
        """If it's a ctypes value, pass on, otherwise do asArray"""
        try:
            return ctypes.byref(value)
        except TypeError as err:
            err.args += (' If you have ERROR_ON_COPY enabled, remember to pass in an array to array-requiring functions.', )
            raise
    dataPointer = from_param
    def zeros( self, dims, typeCode=None ):
        """Currently don't allow Number as output types!"""
        raise NotImplemented( """Number data-type not allowed as an output array format""" )
    def ones( self, dims, typeCode=None ):
        """Currently don't allow Number as output types!"""
        raise NotImplemented( """Number data-type not allowed as an output array format""" )
    def arrayToGLType( self, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        if value.__class__ in TARGET_TYPES:
            return TARGET_TYPES[ value.__class__ ]
        else:
            guess = DEFAULT_TYPES.get( value.__class__ )
            if guess is not None:
                return guess[1]
            raise TypeError( """Can't guess array data-type for %r types"""%(type(value)))
    def arraySize( self, value, typeCode = None ):
        """Given a data-value, calculate ravelled size for the array"""
        return 1
    def asArray( self, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""

        if value.__class__ in TARGET_TYPES:
            return value
        targetType = CONSTANT_TO_TYPE.get( typeCode )
        if targetType is not None:
            return targetType( value )
        raise TypeError( """Don't know how to convert %r to an array type"""%(
            typeCode,
        ))
    def unitSize( self, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        return 1 # there's only 1 possible value in the set...
    def registerEquivalent( self, typ, base ):
        """Register a sub-class for handling as the base-type"""
        global TARGET_TYPE_TUPLE
        for source in (DEFAULT_TYPES, TARGET_TYPES, BYTE_SIZES):
            if base in source:
                source[typ] = source[base]
        if base in TARGET_TYPES:
            TARGET_TYPE_TUPLE = TARGET_TYPE_TUPLE + (base,)

DEFAULT_TYPES = {
    float: (_types.GLdouble,GL_1_1.GL_DOUBLE),
    int: (_types.GLint,GL_1_1.GL_INT),
    long: (_types.GLint,GL_1_1.GL_INT),
}
TARGET_TYPES = dict([
    (getattr( _types,n),c)
    for (n,c) in _types.ARRAY_TYPE_TO_CONSTANT
])
TARGET_TYPE_TUPLE = tuple([
    getattr(_types,n)
    for (n,c) in _types.ARRAY_TYPE_TO_CONSTANT
])
CONSTANT_TO_TYPE = dict([
    (c,getattr( _types, n))
    for (n,c) in _types.ARRAY_TYPE_TO_CONSTANT
])

BYTE_SIZES = dict([
    ( c, ctypes.sizeof( getattr( _types, n) ) )
    for (n,c) in _types.ARRAY_TYPE_TO_CONSTANT
])

try:
    del n,c
except NameError as err:
    pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_I3D_image_buffer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_I3D_image_buffer',error_checker=_errors._error_checker)
WGL_IMAGE_BUFFER_LOCK_I3D=_C('WGL_IMAGE_BUFFER_LOCK_I3D',0x00000002)
WGL_IMAGE_BUFFER_MIN_ACCESS_I3D=_C('WGL_IMAGE_BUFFER_MIN_ACCESS_I3D',0x00000001)
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.HANDLE),ctypes.POINTER(_cs.LPVOID),ctypes.POINTER(_cs.DWORD),_cs.UINT)
def wglAssociateImageBufferEventsI3D(hDC,pEvent,pAddress,pSize,count):pass
@_f
@_p.types(_cs.LPVOID,_cs.HDC,_cs.DWORD,_cs.UINT)
def wglCreateImageBufferI3D(hDC,dwSize,uFlags):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,_cs.LPVOID)
def wglDestroyImageBufferI3D(hDC,pAddress):pass
@_f
@_p.types(_cs.BOOL,_cs.HDC,ctypes.POINTER(_cs.LPVOID),_cs.UINT)
def wglReleaseImageBufferEventsI3D(hDC,pAddress,count):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_single_precision'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_single_precision',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLclampf)
def glClearDepthfOES(depth):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glClipPlanefOES(plane,equation):pass
@_f
@_p.types(None,_cs.GLclampf,_cs.GLclampf)
def glDepthRangefOES(n,f):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glFrustumfOES(l,r,b,t,n,f):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glGetClipPlanefOES(plane,equation):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glOrthofOES(l,r,b,t,n,f):pass



'''OpenGL extension EXT.compiled_vertex_array

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.compiled_vertex_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines an interface which allows static vertex array
	data to be cached or pre-compiled for more efficient rendering.  This
	is useful for implementations which can cache the transformed results
	of array data for reuse by several DrawArrays, ArrayElement, or
	DrawElements commands.  It is also useful for implementations which
	can transfer array data to fast memory for more efficient processing.
	
	For example, rendering an M by N mesh of quadrilaterals can be
	accomplished by setting up vertex arrays containing all of the
	vertexes in the mesh and issuing M DrawElements commands each of
	which operate on 2 * N vertexes.  Each DrawElements command after
	the first will share N vertexes with the preceding DrawElements
	command.  If the vertex array data is locked while the DrawElements
	commands are executed, then OpenGL may be able to transform each
	of these shared vertexes just once.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/compiled_vertex_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.compiled_vertex_array import *
from OpenGL.raw.GL.EXT.compiled_vertex_array import _EXTENSION_NAME

def glInitCompiledVertexArrayEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Darwin (MacOSX)-specific platform features

This was implemented with the help of the following links:
[1] Apple's Mac OS X OpenGL interfaces: http://developer.apple.com/qa/qa2001/qa1269.html
[2] As above, but updated: http://developer.apple.com/documentation/GraphicsImaging/Conceptual/OpenGL-MacProgGuide/opengl_pg_concepts/chapter_2_section_3.html
[3] CGL reference: http://developer.apple.com/documentation/GraphicsImaging/Reference/CGL_OpenGL/index.html#//apple_ref/doc/uid/TP40001186
[4] Intro to OpenGL on Mac OS X: http://developer.apple.com/documentation/GraphicsImaging/Conceptual/OpenGL-MacProgGuide/opengl_intro/chapter_1_section_1.html#//apple_ref/doc/uid/TP40001987-CH207-TP9

About the  CGL API, (from [1]):
CGL or Core OpenGL is the lowest accessible interface API for OpenGL. 
It knows nothing about windowing systems but can be used directly to 
find both renderer information and as a full screen or off screen 
interface. It is accessible from both Cocoa and Carbon and is what both 
NSGL and AGL are built on. A complete Pbuffer interface is also provided. 
Functionality is provided in via the OpenGL framework and applications 
can include the OpenGL.h header to access CGL's functionality. Developers
can see an example of using CGL with Carbon in the Carbon CGL code sample.

Documentation and header files are found in:
/System/Library/Frameworks/OpenGL.framework
/System/Library/Frameworks/GLUT.framework

"""
import ctypes, ctypes.util
from OpenGL.platform import baseplatform, ctypesloader

class DarwinPlatform( baseplatform.BasePlatform ):
    """Darwin (OSX) platform implementation"""
    DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.CFUNCTYPE )
    EXTENSIONS_USE_BASE_FUNCTIONS = True

    @baseplatform.lazy_property
    def GL(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'OpenGL', 
                mode=ctypes.RTLD_GLOBAL 
            ) 
        except OSError as err:
            raise ImportError("Unable to load OpenGL library", *err.args)
    @baseplatform.lazy_property
    def GLU(self): return self.GL
    @baseplatform.lazy_property
    def CGL(self): return self.GL

    @baseplatform.lazy_property
    def GLUT( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLUT', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def GLE(self): return self.GLUT

    @baseplatform.lazy_property
    def GetCurrentContext( self ):
        return self.CGL.CGLGetCurrentContext 

    def getGLUTFontPointer( self, constant ):
        """Platform specific function to retrieve a GLUT font pointer
        
        GLUTAPI void *glutBitmap9By15;
        #define GLUT_BITMAP_9_BY_15     (&glutBitmap9By15)
        
        Key here is that we want the addressof the pointer in the DLL,
        not the pointer in the DLL.  That is, our pointer is to the 
        pointer defined in the DLL, we don't want the *value* stored in
        that pointer.
        """
        name = [ x.title() for x in constant.split( '_' )[1:] ]
        internal = 'glut' + "".join( [x.title() for x in name] )
        pointer = ctypes.c_void_p.in_dll( self.GLUT, internal )
        return ctypes.c_void_p(ctypes.addressof(pointer))

    @baseplatform.lazy_property
    def glGetError( self ):
        return self.GL.glGetError




'''OpenGL extension ARB.vertex_program

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.vertex_program to provide a more 
Python-friendly API

Overview (from the spec)
	
	Unextended OpenGL mandates a certain set of configurable per-vertex
	computations defining vertex transformation, texture coordinate generation
	and transformation, and lighting.  Several extensions have added further
	per-vertex computations to OpenGL.  For example, extensions have defined
	new texture coordinate generation modes (ARB_texture_cube_map,
	NV_texgen_reflection, NV_texgen_emboss), new vertex transformation modes
	(ARB_vertex_blend, EXT_vertex_weighting), new lighting modes (OpenGL 1.2's
	separate specular and rescale normal functionality), several modes for fog
	distance generation (NV_fog_distance), and eye-distance point size
	attenuation (EXT/ARB_point_parameters).
	
	Each such extension adds a small set of relatively inflexible
	per-vertex computations.
	
	This inflexibility is in contrast to the typical flexibility provided by
	the underlying programmable floating point engines (whether micro-coded
	vertex engines, DSPs, or CPUs) that are traditionally used to implement
	OpenGL's per-vertex computations.  The purpose of this extension is to
	expose to the OpenGL application writer a significant degree of per-vertex
	programmability for computing vertex parameters.
	
	For the purposes of discussing this extension, a vertex program is a
	sequence of floating-point 4-component vector operations that determines
	how a set of program parameters (defined outside of OpenGL's Begin/End
	pair) and an input set of per-vertex parameters are transformed to a set
	of per-vertex result parameters.
	
	The per-vertex computations for standard OpenGL given a particular set of
	lighting and texture coordinate generation modes (along with any state for
	extensions defining per-vertex computations) is, in essence, a vertex
	program.  However, the sequence of operations is defined implicitly by the
	current OpenGL state settings rather than defined explicitly as a sequence
	of instructions.
	
	This extension provides an explicit mechanism for defining vertex program
	instruction sequences for application-defined vertex programs.  In order
	to define such vertex programs, this extension defines a vertex
	programming model including a floating-point 4-component vector
	instruction set and a relatively large set of floating-point 4-component
	registers.
	
	The extension's vertex programming model is designed for efficient
	hardware implementation and to support a wide variety of vertex programs.
	By design, the entire set of existing vertex programs defined by existing
	OpenGL per-vertex computation extensions can be implemented using the
	extension's vertex programming model.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/vertex_program.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.vertex_program import *
from OpenGL.raw.GL.ARB.vertex_program import _EXTENSION_NAME

def glInitVertexProgramARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexAttrib1dvARB=wrapper.wrapper(glVertexAttrib1dvARB).setInputArraySize(
    'v', 1
)
glVertexAttrib1fvARB=wrapper.wrapper(glVertexAttrib1fvARB).setInputArraySize(
    'v', 1
)
glVertexAttrib1svARB=wrapper.wrapper(glVertexAttrib1svARB).setInputArraySize(
    'v', 1
)
glVertexAttrib2dvARB=wrapper.wrapper(glVertexAttrib2dvARB).setInputArraySize(
    'v', 2
)
glVertexAttrib2fvARB=wrapper.wrapper(glVertexAttrib2fvARB).setInputArraySize(
    'v', 2
)
glVertexAttrib2svARB=wrapper.wrapper(glVertexAttrib2svARB).setInputArraySize(
    'v', 2
)
glVertexAttrib3dvARB=wrapper.wrapper(glVertexAttrib3dvARB).setInputArraySize(
    'v', 3
)
glVertexAttrib3fvARB=wrapper.wrapper(glVertexAttrib3fvARB).setInputArraySize(
    'v', 3
)
glVertexAttrib3svARB=wrapper.wrapper(glVertexAttrib3svARB).setInputArraySize(
    'v', 3
)
glVertexAttrib4NbvARB=wrapper.wrapper(glVertexAttrib4NbvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NivARB=wrapper.wrapper(glVertexAttrib4NivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NsvARB=wrapper.wrapper(glVertexAttrib4NsvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NubvARB=wrapper.wrapper(glVertexAttrib4NubvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NuivARB=wrapper.wrapper(glVertexAttrib4NuivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4NusvARB=wrapper.wrapper(glVertexAttrib4NusvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4bvARB=wrapper.wrapper(glVertexAttrib4bvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4dvARB=wrapper.wrapper(glVertexAttrib4dvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4fvARB=wrapper.wrapper(glVertexAttrib4fvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4ivARB=wrapper.wrapper(glVertexAttrib4ivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4svARB=wrapper.wrapper(glVertexAttrib4svARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4ubvARB=wrapper.wrapper(glVertexAttrib4ubvARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4uivARB=wrapper.wrapper(glVertexAttrib4uivARB).setInputArraySize(
    'v', 4
)
glVertexAttrib4usvARB=wrapper.wrapper(glVertexAttrib4usvARB).setInputArraySize(
    'v', 4
)
# INPUT glVertexAttribPointerARB.pointer size not checked against 'size,type,stride'
glVertexAttribPointerARB=wrapper.wrapper(glVertexAttribPointerARB).setInputArraySize(
    'pointer', None
)
# INPUT glProgramStringARB.string size not checked against len
glProgramStringARB=wrapper.wrapper(glProgramStringARB).setInputArraySize(
    'string', None
)
# INPUT glDeleteProgramsARB.programs size not checked against n
glDeleteProgramsARB=wrapper.wrapper(glDeleteProgramsARB).setInputArraySize(
    'programs', None
)
glGenProgramsARB=wrapper.wrapper(glGenProgramsARB).setOutput(
    'programs',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glProgramEnvParameter4dvARB=wrapper.wrapper(glProgramEnvParameter4dvARB).setInputArraySize(
    'params', 4
)
glProgramEnvParameter4fvARB=wrapper.wrapper(glProgramEnvParameter4fvARB).setInputArraySize(
    'params', 4
)
glProgramLocalParameter4dvARB=wrapper.wrapper(glProgramLocalParameter4dvARB).setInputArraySize(
    'params', 4
)
glProgramLocalParameter4fvARB=wrapper.wrapper(glProgramLocalParameter4fvARB).setInputArraySize(
    'params', 4
)
glGetProgramEnvParameterdvARB=wrapper.wrapper(glGetProgramEnvParameterdvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramEnvParameterfvARB=wrapper.wrapper(glGetProgramEnvParameterfvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramLocalParameterdvARB=wrapper.wrapper(glGetProgramLocalParameterdvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramLocalParameterfvARB=wrapper.wrapper(glGetProgramLocalParameterfvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramivARB=wrapper.wrapper(glGetProgramivARB).setOutput(
    'params',size=(1,),orPassIn=True
)
# OUTPUT glGetProgramStringARB.string COMPSIZE(target, pname) 
glGetVertexAttribdvARB=wrapper.wrapper(glGetVertexAttribdvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribfvARB=wrapper.wrapper(glGetVertexAttribfvARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribivARB=wrapper.wrapper(glGetVertexAttribivARB).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribPointervARB=wrapper.wrapper(glGetVertexAttribPointervARB).setOutput(
    'pointer',size=(1,),orPassIn=True
)
### END AUTOGENERATED SECTION
from OpenGL.lazywrapper import lazy as _lazy

from OpenGL import converters, error, contextdata
from OpenGL.arrays.arraydatatype import ArrayDatatype
# Note: sizes here are == the only documented sizes I could find,
# may need a lookup table some day...

@_lazy( glVertexAttribPointerARB )
def glVertexAttribPointerARB( 
    baseOperation, index, size, type,
    normalized, stride, pointer,
):
    """Set an attribute pointer for a given shader (index)
    
    index -- the index of the generic vertex to bind, see 
        glGetAttribLocation for retrieval of the value,
        note that index is a global variable, not per-shader
    size -- number of basic elements per record, 1,2,3, or 4
    type -- enum constant for data-type 
    normalized -- whether to perform int to float 
        normalization on integer-type values
    stride -- stride in machine units (bytes) between 
        consecutive records, normally used to create 
        "interleaved" arrays 
    pointer -- data-pointer which provides the data-values,
        normally a vertex-buffer-object or offset into the 
        same.
    
    This implementation stores a copy of the data-pointer 
    in the contextdata structure in order to prevent null-
    reference errors in the renderer.
    """
    array = ArrayDatatype.asArray( pointer, type )
    key = ('vertex-attrib',index)
    contextdata.setValue( key, array )
    return baseOperation(
        index, size, type,
        normalized, stride, 
        ArrayDatatype.voidDataPointer( array ) 
    )



'''OpenGL extension ARB.fragment_program_shadow

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.fragment_program_shadow to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension extends ARB_fragment_program to remove
	the interaction with ARB_shadow.
	
	This extension defines the program option
	"ARB_fragment_program_shadow".
	
	If a fragment program specifies the option "ARB_fragment_program_shadow"
	
	    SHADOW1D, SHADOW2D, SHADOWRECT
	
	are added as texture targets.  When shadow map comparisons are
	desired, specify the SHADOW1D, SHADOW2D, or SHADOWRECT texture
	targets in texture instructions.
	
	Programs must assure that the comparison mode for each depth
	texture (TEXTURE_COMPARE_MODE) and/or the internal texture
	format (DEPTH_COMPONENT) and the targets of the texture lookup
	instructions match.  Otherwise, if the comparison mode
	and/or the internal texture format are inconsistent with the
	texture target, the results of the texture lookup are undefined.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/fragment_program_shadow.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.fragment_program_shadow import *
from OpenGL.raw.GL.ARB.fragment_program_shadow import _EXTENSION_NAME

def glInitFragmentProgramShadowARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION

# This extension is entirely within the fragment program functionality,
# it doesn't affect the function-level operations AFAICS.


'''OpenGL extension EXT.texture_compression_latc

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.texture_compression_latc to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces four new block-based texture compression
	formats suited for unsigned and signed luminance and luminance-alpha
	textures (hence the name "latc" for Luminance-Alpha Texture
	Compression).
	
	These formats are designed to reduce the storage requirements and
	memory bandwidth required for luminance and luminance-alpha textures
	by a factor of 2-to-1 over conventional uncompressed luminance and
	luminance-alpha textures with 8-bit components (GL_LUMINANCE8 and
	GL_LUMINANCE8_ALPHA8).
	
	The compressed signed luminance-alpha format is reasonably suited
	for storing compressed normal maps.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_compression_latc.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.texture_compression_latc import *
from OpenGL.raw.GL.EXT.texture_compression_latc import _EXTENSION_NAME

def glInitTextureCompressionLatcEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_OES_sample_variables'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_OES_sample_variables',error_checker=_errors._error_checker)





'''OpenGL extension AMD.draw_buffers_blend

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.draw_buffers_blend to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds upon the ARB_draw_buffers and EXT_draw_buffers2
	extensions. In ARB_draw_buffers (part of OpenGL 2.0), separate values
	could be written to each color buffer.  This was further enhanced by
	EXT_draw_buffers2 by adding in the ability to enable blending and to set
	color write masks independently per color output.
	
	This extension provides the ability to set individual blend equations and
	blend functions for each color output.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/draw_buffers_blend.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.draw_buffers_blend import *
from OpenGL.raw.GL.AMD.draw_buffers_blend import _EXTENSION_NAME

def glInitDrawBuffersBlendAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_1_4'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_1_4',error_checker=_errors._error_checker)
GL_BLEND_DST_ALPHA=_C('GL_BLEND_DST_ALPHA',0x80CA)
GL_BLEND_DST_RGB=_C('GL_BLEND_DST_RGB',0x80C8)
GL_BLEND_SRC_ALPHA=_C('GL_BLEND_SRC_ALPHA',0x80CB)
GL_BLEND_SRC_RGB=_C('GL_BLEND_SRC_RGB',0x80C9)
GL_COLOR_SUM=_C('GL_COLOR_SUM',0x8458)
GL_COMPARE_R_TO_TEXTURE=_C('GL_COMPARE_R_TO_TEXTURE',0x884E)
GL_CONSTANT_ALPHA=_C('GL_CONSTANT_ALPHA',0x8003)
GL_CONSTANT_COLOR=_C('GL_CONSTANT_COLOR',0x8001)
GL_CURRENT_FOG_COORDINATE=_C('GL_CURRENT_FOG_COORDINATE',0x8453)
GL_CURRENT_SECONDARY_COLOR=_C('GL_CURRENT_SECONDARY_COLOR',0x8459)
GL_DECR_WRAP=_C('GL_DECR_WRAP',0x8508)
GL_DEPTH_COMPONENT16=_C('GL_DEPTH_COMPONENT16',0x81A5)
GL_DEPTH_COMPONENT24=_C('GL_DEPTH_COMPONENT24',0x81A6)
GL_DEPTH_COMPONENT32=_C('GL_DEPTH_COMPONENT32',0x81A7)
GL_DEPTH_TEXTURE_MODE=_C('GL_DEPTH_TEXTURE_MODE',0x884B)
GL_FOG_COORDINATE=_C('GL_FOG_COORDINATE',0x8451)
GL_FOG_COORDINATE_ARRAY=_C('GL_FOG_COORDINATE_ARRAY',0x8457)
GL_FOG_COORDINATE_ARRAY_POINTER=_C('GL_FOG_COORDINATE_ARRAY_POINTER',0x8456)
GL_FOG_COORDINATE_ARRAY_STRIDE=_C('GL_FOG_COORDINATE_ARRAY_STRIDE',0x8455)
GL_FOG_COORDINATE_ARRAY_TYPE=_C('GL_FOG_COORDINATE_ARRAY_TYPE',0x8454)
GL_FOG_COORDINATE_SOURCE=_C('GL_FOG_COORDINATE_SOURCE',0x8450)
GL_FRAGMENT_DEPTH=_C('GL_FRAGMENT_DEPTH',0x8452)
GL_FUNC_ADD=_C('GL_FUNC_ADD',0x8006)
GL_FUNC_REVERSE_SUBTRACT=_C('GL_FUNC_REVERSE_SUBTRACT',0x800B)
GL_FUNC_SUBTRACT=_C('GL_FUNC_SUBTRACT',0x800A)
GL_GENERATE_MIPMAP=_C('GL_GENERATE_MIPMAP',0x8191)
GL_GENERATE_MIPMAP_HINT=_C('GL_GENERATE_MIPMAP_HINT',0x8192)
GL_INCR_WRAP=_C('GL_INCR_WRAP',0x8507)
GL_MAX=_C('GL_MAX',0x8008)
GL_MAX_TEXTURE_LOD_BIAS=_C('GL_MAX_TEXTURE_LOD_BIAS',0x84FD)
GL_MIN=_C('GL_MIN',0x8007)
GL_MIRRORED_REPEAT=_C('GL_MIRRORED_REPEAT',0x8370)
GL_ONE_MINUS_CONSTANT_ALPHA=_C('GL_ONE_MINUS_CONSTANT_ALPHA',0x8004)
GL_ONE_MINUS_CONSTANT_COLOR=_C('GL_ONE_MINUS_CONSTANT_COLOR',0x8002)
GL_POINT_DISTANCE_ATTENUATION=_C('GL_POINT_DISTANCE_ATTENUATION',0x8129)
GL_POINT_FADE_THRESHOLD_SIZE=_C('GL_POINT_FADE_THRESHOLD_SIZE',0x8128)
GL_POINT_SIZE_MAX=_C('GL_POINT_SIZE_MAX',0x8127)
GL_POINT_SIZE_MIN=_C('GL_POINT_SIZE_MIN',0x8126)
GL_SECONDARY_COLOR_ARRAY=_C('GL_SECONDARY_COLOR_ARRAY',0x845E)
GL_SECONDARY_COLOR_ARRAY_POINTER=_C('GL_SECONDARY_COLOR_ARRAY_POINTER',0x845D)
GL_SECONDARY_COLOR_ARRAY_SIZE=_C('GL_SECONDARY_COLOR_ARRAY_SIZE',0x845A)
GL_SECONDARY_COLOR_ARRAY_STRIDE=_C('GL_SECONDARY_COLOR_ARRAY_STRIDE',0x845C)
GL_SECONDARY_COLOR_ARRAY_TYPE=_C('GL_SECONDARY_COLOR_ARRAY_TYPE',0x845B)
GL_TEXTURE_COMPARE_FUNC=_C('GL_TEXTURE_COMPARE_FUNC',0x884D)
GL_TEXTURE_COMPARE_MODE=_C('GL_TEXTURE_COMPARE_MODE',0x884C)
GL_TEXTURE_DEPTH_SIZE=_C('GL_TEXTURE_DEPTH_SIZE',0x884A)
GL_TEXTURE_FILTER_CONTROL=_C('GL_TEXTURE_FILTER_CONTROL',0x8500)
GL_TEXTURE_LOD_BIAS=_C('GL_TEXTURE_LOD_BIAS',0x8501)
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glBlendColor(red,green,blue,alpha):pass
@_f
@_p.types(None,_cs.GLenum)
def glBlendEquation(mode):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glBlendFuncSeparate(sfactorRGB,dfactorRGB,sfactorAlpha,dfactorAlpha):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glFogCoordPointer(type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLdouble)
def glFogCoordd(coord):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glFogCoorddv(coord):pass
@_f
@_p.types(None,_cs.GLfloat)
def glFogCoordf(coord):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glFogCoordfv(coord):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray,arrays.GLsizeiArray,_cs.GLsizei)
def glMultiDrawArrays(mode,first,count,drawcount):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLsizeiArray,_cs.GLenum,arrays.GLvoidpArray,_cs.GLsizei)
def glMultiDrawElements(mode,count,type,indices,drawcount):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat)
def glPointParameterf(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glPointParameterfv(pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glPointParameteri(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glPointParameteriv(pname,params):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glSecondaryColor3b(red,green,blue):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glSecondaryColor3bv(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glSecondaryColor3d(red,green,blue):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glSecondaryColor3dv(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glSecondaryColor3f(red,green,blue):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glSecondaryColor3fv(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glSecondaryColor3i(red,green,blue):pass
@_f
@_p.types(None,arrays.GLintArray)
def glSecondaryColor3iv(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glSecondaryColor3s(red,green,blue):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glSecondaryColor3sv(v):pass
@_f
@_p.types(None,_cs.GLubyte,_cs.GLubyte,_cs.GLubyte)
def glSecondaryColor3ub(red,green,blue):pass
@_f
@_p.types(None,arrays.GLubyteArray)
def glSecondaryColor3ubv(v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glSecondaryColor3ui(red,green,blue):pass
@_f
@_p.types(None,arrays.GLuintArray)
def glSecondaryColor3uiv(v):pass
@_f
@_p.types(None,_cs.GLushort,_cs.GLushort,_cs.GLushort)
def glSecondaryColor3us(red,green,blue):pass
@_f
@_p.types(None,arrays.GLushortArray)
def glSecondaryColor3usv(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glSecondaryColorPointer(size,type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble)
def glWindowPos2d(x,y):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos2dv(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glWindowPos2f(x,y):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos2fv(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint)
def glWindowPos2i(x,y):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos2iv(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort)
def glWindowPos2s(x,y):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos2sv(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glWindowPos3d(x,y,z):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glWindowPos3dv(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glWindowPos3f(x,y,z):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glWindowPos3fv(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glWindowPos3i(x,y,z):pass
@_f
@_p.types(None,arrays.GLintArray)
def glWindowPos3iv(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glWindowPos3s(x,y,z):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glWindowPos3sv(v):pass



'''OpenGL extension ARB.vertex_array_object

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.vertex_array_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces named vertex array objects which encapsulate
	vertex array state on the client side.  These objects allow applications
	to rapidly switch between large sets of array state.  In addition, layered
	libraries can return to the default array state by simply creating and
	binding a new vertex array object.
	
	This extension differs from GL_APPLE_vertex_array_object in that client
	memory cannot be accessed through a non-zero vertex array object.  It also
	differs in that vertex array objects are explicitly not sharable between
	contexts.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/vertex_array_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.vertex_array_object import *
from OpenGL.raw.GL.ARB.vertex_array_object import _EXTENSION_NAME

def glInitVertexArrayObjectARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDeleteVertexArrays.arrays size not checked against n
glDeleteVertexArrays=wrapper.wrapper(glDeleteVertexArrays).setInputArraySize(
    'arrays', None
)
glGenVertexArrays=wrapper.wrapper(glGenVertexArrays).setOutput(
    'arrays',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
### END AUTOGENERATED SECTION

glGenVertexArrays = wrapper.wrapper(glGenVertexArrays).setOutput(
    'arrays', lambda n, array=None: (n,), 'n', arrayType = arrays.GLuintArray,
    orPassIn=True,
)
